#ifndef _BLOG_PLUGIN_H_
#define _BLOG_PLUGIN_H_

#include <glib.h>

typedef struct _BlogPlugin BlogPlugin;

#include "blog.h"

typedef struct _BlogPluginError BlogPluginError;
struct _BlogPluginError {
	gint error_code;
	const gchar *error_string;
};

typedef void (*BlogConnectedCb)(Blog **blogs, const BlogPluginError *error, gpointer user_data);
typedef void (*BlogFetchCb)(Blog *blog, const BlogPluginError *error, gpointer user_data);
typedef void (*BlogPostCb)(Blog *blog, BlogPost *post, gboolean status, const BlogPluginError *error, gpointer user_data);
typedef void (*BlogUploadCb)(Blog *blog, const gchar *image_name, const gchar *image_path, const BlogPluginError *error, gpointer user_data);

struct _BlogPlugin {
	gboolean (*connect)(BlogPlugin *plugin,
			const gchar *uri,
			const gchar *user,
		       	const gchar *pass,
			BlogConnectedCb cb,
			gpointer user_data);
	gboolean (*newPost)(Blog *blog,
		       	BlogPost *post,
		       	gboolean publish,
		       	BlogPostCb cb,
		       	gpointer user_data);
	gboolean (*editPost)(Blog *blog,
		       	BlogPost *post,
		       	gboolean publish,
		       	BlogPostCb cb,
		       	gpointer user_data);
	void (*getCategories)(Blog *blog,
		       	BlogFetchCb cb,
		       	gpointer user_data);
	void (*getRecent)(Blog *blog,
		       	BlogFetchCb cb,
		       	gpointer user_data);
	void (*unLoad)(BlogPlugin *plugin);
	gboolean (*uploadImage)(Blog *blog,
		       	const gchar *image_name, 
			const gchar *mime_type,
			const gchar *image_data,
			size_t data_size,
			BlogUploadCb cb,
			gpointer user_data);
	char *name;

	void *plugin_data;
	void *plugin_handle;
};

BlogPlugin *plugin_init(void);

#endif /* _BLOG_PLUGIN_H_ */
