#include "blog.h"

static void blog_free(Blog *blog);

void blog_parameter_free(BlogParameter *param)
{
	g_free(param->name);
	g_value_unset(&param->value);
	g_free(param);
}

void blog_post_free(BlogPost *post)
{
	g_free(post->id);
	g_free(post->title);
	g_free(post->content);
	g_slist_foreach(post->extra_fields, (GFunc)blog_parameter_free, NULL);
	g_free(post);
}

void blog_category_free(BlogCategory *category)
{
	g_free(category->name);
	g_free(category->id);
	g_free(category);
}

Blog *blog_new() {
	Blog *retval = g_new0(Blog, 1);
	retval->refcount = 1;
	return retval;
}

void blog_ref(Blog *blog) {
	blog->refcount++;
}

void blog_unref(Blog *blog)
{
	if (!(--blog->refcount)) {
		blog_free(blog);
	}
}

static void blog_free(Blog *blog)
{
	g_free(blog->url);
	g_free(blog->user);
	g_free(blog->pass);
	g_free(blog->id);
	g_free(blog->name);
	g_slist_foreach(blog->categories, (GFunc)blog_category_free, NULL);
	g_slist_free(blog->categories);
	g_slist_foreach(blog->posts, (GFunc)blog_post_free, NULL);
	g_slist_free(blog->posts);
}
