#include <string.h>
#ifdef HILDON_FM
#include <hildon-widgets/hildon-file-chooser-dialog.h>
#endif
#ifdef HILDON
#include <hildon-widgets/hildon-window.h>
#include <hildon-widgets/hildon-program.h>
#endif
#include <gtk/gtk.h>
#ifdef GTKSOURCEVIEW
#include <gtksourceview/gtksourceview.h>
#include <gtksourceview/gtksourcelanguagesmanager.h>
#endif
#ifdef HAVE_VFS_UTIL
#include <libgnomeui/gnome-vfs-util.h>
#endif
#ifdef GEOCLUE
#include <geoclue/position.h>
#endif
#include "blog-plugin.h"
#include "gui.h"

#include <stdlib.h>

struct UploadClosure {
	GMainLoop *loop;
	gchar *name;
};

static gboolean _gtk_widget_destroy(gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(data));
	return FALSE;
}

static void _uploaded(Blog *blog, const gchar *image_name, const gchar *image_path, const BlogPluginError *error, gpointer user_data)
{
	struct UploadClosure *closure = user_data;
	(void)blog;
	(void)error;
	(void)image_name;
	closure->name = g_strdup(image_path);
	if (g_main_loop_is_running(closure->loop)) {
		g_main_loop_quit(closure->loop);
	}
}

static gchar *upload_image(Blog *blog,
		const gchar *name,
		const gchar *mime_type,
		const gchar *data,
		const size_t size)
{
	struct UploadClosure closure = { NULL, NULL };

	closure.loop = g_main_loop_new(NULL, FALSE);
	if (blog->plugin->uploadImage(blog,
				name,
				mime_type,
				data,
				size,
				_uploaded,
				&closure)) {
		g_main_loop_run(closure.loop);
	}
	g_main_loop_unref(closure.loop);

	return closure.name;
}

struct PostClosure {
	GMainLoop *loop;
	gboolean status;
};

static void _posted(Blog *blog, BlogPost *post, gboolean status, const BlogPluginError *error, gpointer user_data)
{
	struct PostClosure *closure = user_data;
	(void)blog;
	(void)post;
	(void)error;

	closure->status = status;
	
	if (g_main_loop_is_running(closure->loop)) {
		g_main_loop_quit(closure->loop);
	}
}

static gboolean post_or_edit(Blog *blog, BlogPost *post, gboolean publish)
{
	struct PostClosure closure = { NULL, FALSE };
	closure.loop = g_main_loop_new(NULL, FALSE);
	if (post->id) {
		blog->plugin->editPost(blog, post, publish, _posted, &closure);
	} else {
		blog->plugin->newPost(blog, post, publish, _posted, &closure);
	}
	g_main_loop_run(closure.loop);
	g_main_loop_unref(closure.loop);

	return closure.status;
}

static unsigned int _strcount(const char *haystack, const char *needle)
{
	unsigned int retval = 0;
	unsigned int needlelen = strlen(needle);
	char *iter;
	iter = strstr(haystack, needle);
	while (iter) {
		retval++;
		iter = strstr(iter + needlelen, needle);
	}

	return retval;
}

static void _toggle_tool_button_set_inconsistent(GtkToggleToolButton *button,
		gboolean inconsistent)
{
	GtkWidget *toggle_button = gtk_bin_get_child(GTK_BIN(button));
	gtk_toggle_button_set_inconsistent(GTK_TOGGLE_BUTTON(toggle_button),
			inconsistent);
}

#ifdef HILDON
static void _tool_button_set_icon_name(GtkToolButton *button,
		const gchar *icon_name)
{
	GtkWidget *image;
	image = gtk_image_new_from_icon_name(icon_name,
			GTK_ICON_SIZE_SMALL_TOOLBAR);
	gtk_tool_button_set_icon_widget(button, image);
}
#endif

static void _combo_changed(GtkComboBox *combo, GObject *window)
{
	GtkTextBuffer *source = g_object_get_data(window, "buffer");
	GtkEntry *entry = GTK_ENTRY(g_object_get_data(window, "title-entry"));
	GtkTextIter start, end;
	GtkWidget *item = g_object_get_data(window, "categories");
	GtkWidget *menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
	GList *items = menu ? gtk_container_get_children(GTK_CONTAINER(menu))
		: NULL;
	gint active = gtk_combo_box_get_active(combo);
	BlogPost *post;
	Blog *blog = g_object_get_data(window, "blog");
	GList *iter;

	if (active == -1) {
		g_list_free(items);
		return;
	}

	gtk_text_buffer_get_start_iter(source, &start);
	gtk_text_buffer_get_end_iter(source, &end);
	gtk_text_buffer_delete(source, &start, &end);

	if (active == 0) {
		gtk_entry_set_text(entry, "");
		for (iter = items; iter; iter = iter->next) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
						iter->data), FALSE);
		}
		g_list_free(items);
		return;
	}

	post = (BlogPost *)g_slist_nth_data(blog->posts, active - 1);
	gtk_text_buffer_insert_at_cursor(source,
			post->content, -1);
	gtk_entry_set_text(entry, post->title);
	
	for (iter = items; iter; iter = iter->next) {
		gpointer *data = g_object_get_data(G_OBJECT(iter->data),
				"category");
		if (g_slist_find(post->categories, data)) {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
						iter->data), TRUE);
		} else {
			gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(
						iter->data), FALSE);
		}
	}
	g_list_free(items);
}

static void _more_magic(GtkTextBuffer *source, GtkTextBuffer *text);

static void _img(GObject *object, GtkTextBuffer *text)
{
#ifdef HILDON_FM
	GtkWidget *dialog = hildon_file_chooser_dialog_new(
			GTK_WINDOW(gtk_widget_get_toplevel(
					GTK_WIDGET(object))),
			GTK_FILE_CHOOSER_ACTION_OPEN);
	gtk_window_set_title(GTK_WINDOW(dialog), "Open file...");
	gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			"Ok", GTK_RESPONSE_OK,
			"Cancel", GTK_RESPONSE_CANCEL,
			NULL);
#else
	GtkWidget *dialog = gtk_file_chooser_dialog_new("Open file...",
			GTK_WINDOW(gtk_widget_get_toplevel(
					GTK_WIDGET(object))),
			GTK_FILE_CHOOSER_ACTION_OPEN,
			"Ok", GTK_RESPONSE_OK,
			"Cancel", GTK_RESPONSE_CANCEL,
			NULL);
#endif

	gtk_widget_show_all(GTK_WIDGET(dialog));
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
#ifdef HAVE_VFS_UTIL
		gchar *file = gtk_file_chooser_get_uri(
				GTK_FILE_CHOOSER(dialog));
		GdkPixbuf *buf = gnome_gdk_pixbuf_new_from_uri(file);
#else
		gchar *file = gtk_file_chooser_get_filename(
				GTK_FILE_CHOOSER(dialog));
		GdkPixbuf *buf = gdk_pixbuf_new_from_file(file, NULL);
#endif
		gchar *sep = strrchr(file, '/');
		GObject *source = g_object_get_data(G_OBJECT(text),
				"source-buffer");
		GHashTable *table = g_object_get_data(source, "image-buffer");

		if (sep) {
			sep++;
		} else {
			sep = file;
		}

		g_hash_table_replace(table,
				g_strdup(sep),
				buf);

		GtkTextMark *mark = gtk_text_buffer_get_insert(text);
		GtkTextIter iter;
		GtkTextIter iter2;
		GtkTextTag *tag = gtk_text_buffer_create_tag(text, NULL, NULL);

		g_object_set_data_full(G_OBJECT(tag), "start",
				g_strdup_printf("<img src=\"maemo-blog:%s\" />", sep),
			       	g_free);

		g_free(file);

		gtk_text_buffer_get_iter_at_mark(text, &iter, mark);

		gtk_text_buffer_insert_pixbuf(text, &iter, buf);
		iter2 = iter;
		gtk_text_iter_backward_char(&iter2);
		gtk_text_buffer_apply_tag(text, tag, &iter2, &iter);
		g_signal_emit_by_name(text, "changed");
	}
	gtk_widget_destroy(dialog);
}

#ifdef GEOCLUE
static void _geo(GObject *object, GObject *window)
{
	gdouble lat, lon;

	(void)object;

	geoclue_position_init();
	geoclue_position_current_position(&lat, &lon);
	g_object_set_data_full(window, "position",
			g_strdup_printf("%f %f", lat, lon),
			g_free);
	fprintf(stderr, "Position now: %s\n", (gchar *)g_object_get_data(window, "position"));
}
#endif

static void _send(GObject *object, GObject *window)
{
	Blog *blog = (Blog *)g_object_get_data(window, "blog");
	GtkWidget *item = g_object_get_data(window, "categories");
	GtkWidget *menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(item));
	GList *items = menu ? gtk_container_get_children(GTK_CONTAINER(menu))
		: NULL;
	GtkTextIter start, end;
	GtkTextBuffer *buf;
	gchar *text, *title;
	BlogPost *post;
	GtkWidget *dialog = gtk_dialog_new_with_buttons("Sending...",
			GTK_WINDOW(window),
			GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
			NULL);
	GtkWidget *pbar = gtk_progress_bar_new();
	GtkComboBox *combo = GTK_COMBO_BOX(g_object_get_data(
				window, "combo"));
	gint active;
	GList *iter;
	GHashTable *url_cache = g_hash_table_new_full(g_str_hash,
			g_str_equal,
			g_free,
			g_free);
	GHashTable *image_cache;
	gchar *pos;
	guint image_count, i;
	gchar *position = g_object_get_data(window, "position");

	(void)object;

	g_return_if_fail(blog != NULL);
	active = gtk_combo_box_get_active(combo);
	if (active > 0) {
		post = (BlogPost *)g_slist_nth_data(blog->posts, active - 1);
		g_free(post->content);
		g_free(post->title);
		g_slist_free(post->categories);
		post->categories = NULL;
	} else {
		post = g_new0(BlogPost, 1);
	}

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), pbar);
	gtk_widget_show_all(dialog);

	buf = GTK_TEXT_BUFFER(g_object_get_data(window, "buffer"));
	image_cache = g_object_get_data(G_OBJECT(buf),
			"image-buffer");
	gtk_text_buffer_get_start_iter(buf, &start);
	gtk_text_buffer_get_end_iter(buf, &end);
	text = gtk_text_buffer_get_text(buf, &start, &end, FALSE);
	title = g_strdup(gtk_entry_get_text(GTK_ENTRY(g_object_get_data(
						window, "title-entry"))));
	image_count = _strcount(text, "\"maemo-blog:");
	i = 1;

	while ((pos = strstr(text, "\"maemo-blog:"))) {
		gchar *endquot = strchr(pos + 1, '"');
		gchar *name = g_strndup(pos + 12, endquot - pos - 12);
		gchar *url;
		gchar *new_text;
		gchar *pbar_text = g_strdup_printf("Sending image %d of %d...", i, image_count);
		gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar), pbar_text);
		g_free(pbar_text);
		gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), (gdouble)(i - 1)/(image_count + 1));
		i++;

		while (gtk_events_pending()) {
			gtk_main_iteration();
		}

		if ((url = g_hash_table_lookup(url_cache, "name")));
		else {
			gchar *image_data;
			size_t size;
			GdkPixbuf *pixbuf = g_hash_table_lookup(image_cache,
					name);
			gdk_pixbuf_save_to_buffer(pixbuf, &image_data,
					&size,
					"jpeg",
					NULL,
					"quality",
					"75",
					NULL);
			url = upload_image(blog,
					name,
					"image/jpeg",
					image_data,
					size);
			g_hash_table_insert(url_cache, g_strdup(name),
					url);
		}

		pos[1] = '\0';
		new_text = g_strconcat(text, url, endquot, NULL);
		g_free(text);
		text = new_text;

		break;
	}

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), (gdouble)(i - 1)/(image_count + 1));
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar), "Sending post...");

	while (gtk_events_pending()) {
		gtk_main_iteration();
	}


	post->content = text;
	post->title = title;

	for (iter = items; iter; iter = iter->next) {
		if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(
						iter->data))) {
			gpointer category = 
				g_object_get_data(G_OBJECT(iter->data),
				"category");
			post->categories =
			       	g_slist_append(post->categories, category);
		}
	}

	if (position) {
		BlogParameter *parm = g_new0(BlogParameter, 1);

		parm->name = g_strdup("georss:point");
		g_value_init(&parm->value, G_TYPE_STRING);
		g_value_set_string(&parm->value, position);
		post->extra_fields = g_slist_append(post->extra_fields,
				parm);
	}
	
	if (post->id) {
		post_or_edit(blog, post, TRUE);
	} else if (post_or_edit(blog, post, TRUE)) {
		GtkTextBuffer *dest = GTK_TEXT_BUFFER(g_object_get_data(window, "buffer"));
		GtkTextIter start, end;
		gtk_text_buffer_get_start_iter(dest, &start);
		gtk_text_buffer_get_end_iter(dest, &end);
		gtk_text_buffer_delete(dest, &start, &end);
		gtk_entry_set_text(GTK_ENTRY(g_object_get_data(G_OBJECT(window),
						"title-entry")), "");
		g_signal_emit_by_name(dest, "changed");
	}

	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(pbar), 1.00);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(pbar), "Done.");
	g_timeout_add(200, _gtk_widget_destroy, dialog);
	update_posts(GTK_WIDGET(window), active <= 0 ? 1 : active);
}

static void list_tags(GtkTextTag *tag, gpointer list)
{
	GSList **slist = (GSList **)list;
	*slist = g_slist_append(*slist, tag);
}

static void mark_set(GtkTextBuffer *buffer,
		GtkTextIter *iter,
		GtkTextMark *mark,
		gchar *signal_name)
{
	(void)signal_name;
	if (gtk_text_mark_get_name(mark) &&
			strcmp(gtk_text_mark_get_name(mark),
				"insert") == 0) {
		GSList *tags = NULL;
		gtk_text_tag_table_foreach(gtk_text_buffer_get_tag_table(buffer),
				list_tags, &tags);
		GtkTextIter start, end;
		if (gtk_text_buffer_get_selection_bounds(buffer,
					&start, &end)) {
			GSList *act_tags = gtk_text_iter_get_tags(&start);
			GSList *inc_tags = NULL;
			GSList *iter;
			GtkTextIter tag_tog = start;
			gtk_text_iter_forward_to_tag_toggle(&tag_tog, NULL);
			
			while (gtk_text_iter_compare(&tag_tog, &end) < 0) {
				GSList *tog_tags;
				tog_tags = gtk_text_iter_get_toggled_tags(&tag_tog, TRUE);
				tog_tags = g_slist_concat(tog_tags,
						gtk_text_iter_get_toggled_tags(&tag_tog,
							FALSE));
				iter = tog_tags;
				while (iter) {
					if (g_slist_find(act_tags, iter->data)) {
						act_tags = g_slist_remove(act_tags, iter->data);
					}
					if (g_slist_find(inc_tags, iter->data) == NULL) {
						inc_tags = g_slist_append(inc_tags, iter->data);
					}
					iter = iter->next;
				}
				g_slist_free(tog_tags);
				gtk_text_iter_forward_to_tag_toggle(&tag_tog, NULL);
			}

			for (iter = act_tags; iter; iter = iter->next) {
				gchar *name;
				GtkToggleToolButton *button;

				g_object_get(G_OBJECT(iter->data), "name", &name, NULL);
				if (name) {
				
					if (g_slist_find(tags, iter->data)) {
						tags = g_slist_remove(tags, iter->data);
					}

					button = GTK_TOGGLE_TOOL_BUTTON(
							g_object_get_data(G_OBJECT(buffer), name));
					g_free(name);

					gtk_toggle_tool_button_set_active(button, TRUE);
					_toggle_tool_button_set_inconsistent(button, FALSE);
				}
			}
			g_slist_free(act_tags);

			for (iter = inc_tags; iter; iter = iter->next) {
				gchar *name;
				GtkToggleToolButton *button;

				g_object_get(G_OBJECT(iter->data), "name", &name, NULL);
				if (name) {
					if (g_slist_find(tags, iter->data)) {
						tags = g_slist_remove(tags, iter->data);
					}
					button = GTK_TOGGLE_TOOL_BUTTON(
							g_object_get_data(G_OBJECT(buffer), name));
					g_free(name);

				}
				_toggle_tool_button_set_inconsistent(button, TRUE);
			}
			g_slist_free(inc_tags);

			for (iter = tags; iter; iter = iter->next) {
				gchar *name;
				GtkToggleToolButton *button;

				g_object_get(G_OBJECT(iter->data), "name", &name, NULL);
				if (name) {
				
					button = GTK_TOGGLE_TOOL_BUTTON(
							g_object_get_data(G_OBJECT(buffer), name));
					g_free(name);

					_toggle_tool_button_set_inconsistent(button, FALSE);
					gtk_toggle_tool_button_set_active(button, FALSE);
				}
			}
		} else {
			GtkTextIter itar = *iter;
			gtk_text_iter_backward_char(&itar);
			GSList *act_tags = gtk_text_iter_get_tags(&itar);
			GSList *liter;
			for (liter = act_tags; liter; liter = liter->next) {
				gchar *name;
				GtkToggleToolButton *button;

				g_object_get(G_OBJECT(liter->data), "name", &name, NULL);
				
				if (name) {
					if (g_slist_find(tags, liter->data)) {
						tags = g_slist_remove(tags, liter->data);
					}

					button = GTK_TOGGLE_TOOL_BUTTON(
							g_object_get_data(G_OBJECT(buffer), name));
					g_free(name);

					gtk_toggle_tool_button_set_active(button, TRUE);
					_toggle_tool_button_set_inconsistent(button, FALSE);
				}
			}
			g_slist_free(act_tags);
			for (liter = tags; liter; liter = liter->next) {
				gchar *name;
				GtkToggleToolButton *button;

				g_object_get(G_OBJECT(liter->data), "name", &name, NULL);
				if (name) {
				
					button = GTK_TOGGLE_TOOL_BUTTON(
							g_object_get_data(G_OBJECT(buffer), name));
					g_free(name);

					gtk_toggle_tool_button_set_active(button, FALSE);
					_toggle_tool_button_set_inconsistent(button, FALSE);
				}
			}
			
		}
		g_slist_free(tags);
	}
}

static void _zui(GtkTextBuffer *buffer,
		GtkTextIter *iter,
		gchar *text,
		gint len,
		GtkTextView *view)
{
	GtkTextIter end = *iter;
	GSList *list = (GSList *)g_object_get_data(G_OBJECT(buffer), "tags");

	(void)text;
	(void)view;

	gtk_text_iter_backward_chars(&end, len);

	for (; list; list = list->next) {
		gtk_text_buffer_apply_tag_by_name(buffer,
				list->data,
				&end,
				iter);
	}
	g_signal_emit_by_name(buffer, "changed");
}

static void _omg(GtkToggleToolButton *button, GtkTextView *view)
{
	const gchar *text = g_object_get_data(G_OBJECT(button), "tag");
	GSList *list = (GSList *)g_object_steal_data(G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))), "tags");
	GtkTextIter start, end;

	if (gtk_text_buffer_get_selection_bounds(gtk_text_view_get_buffer(view),
				&start, &end)) {
		if (gtk_toggle_tool_button_get_active(button)) {
			gtk_text_buffer_apply_tag_by_name(gtk_text_view_get_buffer(view),
					text,
					&start,
					&end);
		} else {
			gtk_text_buffer_remove_tag_by_name(gtk_text_view_get_buffer(view),
					text,
					&start,
					&end);
		}
	}
	if (gtk_toggle_tool_button_get_active(button)) {
		list = g_slist_append(list, (gpointer)text);
	} else {
		list = g_slist_remove(list, text);
	}
	_toggle_tool_button_set_inconsistent(button, FALSE);
	g_object_set_data_full(G_OBJECT(gtk_text_view_get_buffer(view)),
		       	"tags", list,
			(GDestroyNotify)g_slist_free);
	g_signal_emit_by_name(gtk_text_view_get_buffer(view), "changed");
}

static void _end_tag(gpointer data, gpointer user_data)
{
	gchar *name;
	g_object_get(G_OBJECT(data), "name", &name, NULL);
	if (name) {
		gchar *temp = g_strdup_printf("</%s>", name);
		guint len = strlen(temp);
		gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(user_data),
				temp, len);
		g_free(name);
		g_free(temp);
	} else {
		gchar *tmp = g_object_get_data(G_OBJECT(data), "end");
		if (tmp) {
			guint len = strlen(tmp);
			gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(user_data),
					tmp, len);
		}
	}
}

static void _start_tag(gpointer data, gpointer user_data)
{
	gchar *name;
	g_object_get(G_OBJECT(data), "name", &name, NULL);
	if (name) {
		gchar *temp = g_strdup_printf("<%s>", name);
		guint len = strlen(temp);
		gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(user_data),
				temp, len);
		g_free(name);
		g_free(temp);
	} else {
		gchar *tmp = g_object_get_data(G_OBJECT(data), "start");
		if (tmp) {
			guint len = strlen(tmp);
			gtk_text_buffer_insert_at_cursor(GTK_TEXT_BUFFER(user_data),
					tmp, len);
		}
	}
}

static void _foo(GtkTextBuffer *buf, GtkTextBuffer *dest)
{
	GtkTextIter start;
	GtkTextIter end;
	GtkTextIter line_end;
	GSList *tags;
	gchar *text;

	gtk_text_buffer_get_start_iter(dest, &start);
	gtk_text_buffer_get_end_iter(dest, &end);
	gtk_text_buffer_delete(dest, &start, &end);

	gtk_text_buffer_insert_at_cursor(dest, "<p>\n ", -1);
	gtk_text_buffer_get_start_iter(buf, &start);
	while (TRUE) {
		tags = gtk_text_iter_get_toggled_tags(&start, FALSE);
		g_slist_foreach(tags, _end_tag, dest);
		g_slist_free(tags);
		tags = gtk_text_iter_get_toggled_tags(&start, TRUE);
		g_slist_foreach(tags, _start_tag, dest);
		g_slist_free(tags);
		end = start;
		line_end = start;
		if (!gtk_text_iter_forward_to_tag_toggle(&end, NULL) ||
			       	!gtk_text_iter_compare(&start, &end))
			break;
		gtk_text_iter_forward_to_line_end(&line_end);
		while (gtk_text_iter_compare(&line_end, &end) <= 0 &&
				gtk_text_iter_compare(&start, &line_end) &&
				!gtk_text_iter_is_end(&line_end)) {
			text = gtk_text_buffer_get_slice(
					buf,
				&start, &line_end, FALSE);
			gtk_text_buffer_insert_at_cursor(dest, text, -1);
			tags = gtk_text_iter_get_tags(&line_end);
			g_slist_foreach(tags, _end_tag, dest);
			gtk_text_buffer_insert_at_cursor(dest, "\n</p>\n<p>\n ", -1);
			g_slist_foreach(tags, _start_tag, dest);
			g_free(text);
			start = line_end;
			gtk_text_iter_forward_char(&start);
			gtk_text_iter_forward_to_line_end(&line_end);
		}
		text = gtk_text_buffer_get_text(
				buf,
				&start, &end, FALSE);
		gtk_text_buffer_insert_at_cursor(dest, text, -1);
		g_free(text);
		start = end;
	}
	
	gtk_text_buffer_get_end_iter(buf, &end);
	gtk_text_iter_forward_to_line_end(&line_end);
	while (gtk_text_iter_compare(&line_end, &end) < 0) {
		text = gtk_text_buffer_get_text(
				buf,
				&start, &line_end, FALSE);
		gtk_text_buffer_insert_at_cursor(dest, text, -1);
		tags = gtk_text_iter_get_tags(&line_end);
		g_slist_foreach(tags, _end_tag, dest);
		gtk_text_buffer_insert_at_cursor(dest, "\n</p>\n<p>\n ", -1);
		g_slist_foreach(tags, _start_tag, dest);
		g_free(text);
		start = line_end;
		gtk_text_iter_forward_char(&start);
		gtk_text_iter_forward_to_line_end(&line_end);
	}
	text = gtk_text_buffer_get_text(
			buf,
			&start,
			&end,
			FALSE);
	gtk_text_buffer_insert_at_cursor(dest, text, -1);
	gtk_text_buffer_insert_at_cursor(dest, "\n</p>", -1);
	

	g_free(text);
	gtk_text_buffer_get_start_iter(buf, &start);
}

static void _magic(GObject *object, GtkTextBuffer *buffer)
{
	GtkTextBuffer *source_buffer = (GtkTextBuffer *)
		g_object_get_data(G_OBJECT(buffer), "source-buffer");

	(void)object;

	g_signal_handlers_block_by_func(source_buffer,
			_more_magic,
			buffer);
	_foo(buffer, source_buffer);
	g_signal_handlers_unblock_by_func(source_buffer,
			_more_magic,
			buffer);
}

static gboolean _match_char(gunichar unic, gpointer match)
{
	return g_utf8_get_char((const gchar *)match) == unic;
}

static void _more_magic(GtkTextBuffer *source, GtkTextBuffer *text)
{
	GtkTextIter iter1;
	GtkTextIter iter2;
	GSList *tags = NULL;
	gboolean in_title = FALSE;

	g_signal_handlers_block_by_func(text,
			_magic,
			text);
	g_signal_handlers_block_by_func(g_object_get_data(G_OBJECT(text),
				"title-entry"),
			_magic,
			text);
	gtk_text_buffer_get_start_iter(text, &iter1);
	gtk_text_buffer_get_end_iter(text, &iter2);
	gtk_text_buffer_delete(text, &iter1, &iter2);

	gtk_text_buffer_get_start_iter(source, &iter1);
	while (TRUE) {
		if (gtk_text_iter_get_char(&iter1) == (gunichar)'<') {
			gchar *tag;
			gboolean end_tag = FALSE;
			gtk_text_iter_forward_char(&iter1);
			iter2 = iter1;
			gtk_text_iter_forward_find_char(&iter2, _match_char,
					(gpointer)">", NULL);
			tag = gtk_text_buffer_get_text(source, &iter1, &iter2, FALSE);
			if (*tag == '/') {
				end_tag = TRUE;
				tag++;
			}
			if (strcmp(tag, "title") == 0) {
				in_title = !end_tag;
				g_free(tag - end_tag);
			} else if (strcmp(tag, "p") == 0) {
				g_free(tag - end_tag);
			} else if (g_str_has_prefix(tag, "img ")) {
				gchar *src = strstr(tag, "src=\"maemo-blog:");

				if (src) {
					GHashTable *table =
						g_object_get_data(
								G_OBJECT(
									source),
								"image-buffer");
					GdkPixbuf *buf;
					gchar *end;
					src += 16;
					end = strchr(src, '"');
					src = g_strndup(src, (end - src));

					buf = g_hash_table_lookup(table,
							src);

					if (buf) {
						GtkTextTag *tagi = gtk_text_buffer_create_tag(text, NULL, NULL);
						GtkTextMark *mark = gtk_text_buffer_get_insert(text);
						GtkTextIter my_iter1, my_iter2;

						g_object_set_data_full(G_OBJECT(tagi), "start",
								g_strdup_printf("<%s>", tag),
								g_free);

						gtk_text_buffer_get_iter_at_mark(text, &my_iter1, mark);

						gtk_text_buffer_insert_pixbuf(text, &my_iter1, buf);
						my_iter2 = my_iter1;
						gtk_text_iter_backward_char(&my_iter1);
						gtk_text_buffer_apply_tag(text, tagi, &my_iter1, &my_iter2);
						
					}

				}
				
			} else {
				if (end_tag) {
					GSList *in_stack =
						g_slist_find_custom(tags,
								tag,
								(GCompareFunc)
								strcmp);
					if (in_stack) {
						g_free(in_stack->data);
						tags = g_slist_delete_link(tags,
								in_stack);
					}
					g_free(tag - end_tag);
				} else {
					tags = g_slist_append(tags, tag);
				}
			}
			iter1 = iter2;
			gtk_text_iter_forward_char(&iter1);
			if (gtk_text_iter_is_end(&iter1))
				break;
		} else {
			gchar *txt;

			iter2 = iter1;
			gtk_text_iter_forward_find_char(&iter2,
					_match_char, (gpointer)"<", NULL);
			txt = gtk_text_buffer_get_text(source, &iter1, &iter2, FALSE);
			g_strstrip(txt);
			if (in_title) {
				gtk_entry_set_text((GtkEntry *)
						g_object_get_data(
							G_OBJECT(text),
							"title-entry"),
						txt);
			} else {
				GtkTextIter diter1;
				GtkTextIter diter2;
				GSList *iter;

				gtk_text_buffer_get_end_iter(text, &diter1);
				gtk_text_buffer_insert(text, &diter1, txt, -1);
				diter2 = diter1;
				gtk_text_iter_backward_chars(&diter2, strlen(txt));
				for (iter = tags; iter; iter = iter->next) {
					gtk_text_buffer_apply_tag_by_name(
							text,
							(gchar *)iter->data,
							&diter1,
							&diter2);
				}
			}
				
			g_free(txt);
			iter1 = iter2;

			if (gtk_text_iter_is_end(&iter1))
				break;
		}
	}
	g_slist_foreach(tags, (GFunc)g_free, NULL);
	g_slist_free(tags);
	g_signal_handlers_unblock_by_func(text,
			_magic,
			text);
	g_signal_handlers_unblock_by_func(g_object_get_data(G_OBJECT(text),
				"title-entry"),
			_magic,
			text);
}

GtkWidget *build_ui()
{
#ifdef HILDON
	GtkWidget *window = hildon_window_new();
#else
	GtkWidget *window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
#endif
	GtkWidget *notebook = gtk_notebook_new();
	GtkWidget *vbox = gtk_vbox_new(0, FALSE);
	GtkWidget *view = gtk_text_view_new();
	GtkWidget *toolbar = gtk_toolbar_new();
	GtkToolItem *button;
	GtkToolItem *combo_item = gtk_tool_item_new();
	GtkWidget *combo = gtk_combo_box_new_text();
	GtkWidget *source_view;
#ifdef HILDON
	GtkWidget *menu = gtk_menu_new();
#else
	GtkWidget *menu = gtk_menu_bar_new();
#endif
#ifdef GTKSOURCEVIEW
	GtkSourceLanguagesManager *manager = (GtkSourceLanguagesManager *)
		g_object_new(GTK_TYPE_SOURCE_LANGUAGES_MANAGER,
				"lang_files_dirs", g_slist_append(NULL,
					g_strdup("/usr/share/gtksourceview-1.0/language-specs")), NULL);
	GtkSourceLanguage *language =
		gtk_source_languages_manager_get_language_from_mime_type(
				manager, "text/html");
	GtkSourceBuffer *buffer = gtk_source_buffer_new_with_language(language);
#else
	GtkTextBuffer *buffer = gtk_text_buffer_new(NULL);
#endif
	GtkWidget *entry = gtk_entry_new();
	GtkWidget *menu_item = NULL;
	GHashTable *image_buffer =
		g_hash_table_new_full(g_str_hash,
				g_str_equal,
				g_free,
				g_object_unref);
	g_object_set_data_full(G_OBJECT(buffer), "image-buffer", image_buffer,
#ifdef HILDON
			(GDestroyNotify)g_hash_table_destroy);
#else
			(GDestroyNotify)g_hash_table_unref);
#endif

	menu_item = gtk_menu_item_new_with_label("Categories");
	gtk_widget_set_sensitive(menu_item, FALSE);
	gtk_menu_shell_append(GTK_MENU_SHELL(menu),
			menu_item);
	g_object_set_data(G_OBJECT(window), "categories", menu_item);

#ifdef HILDON
	hildon_window_set_menu(HILDON_WINDOW(window), GTK_MENU(menu));
#else
	gtk_box_pack_start(GTK_BOX(vbox), menu, FALSE, FALSE, 0);
#endif

#ifdef HILDON
	g_set_prgname("Maemo Blog");
	hildon_program_add_window(hildon_program_get_instance(),
			HILDON_WINDOW(window));
#endif

	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_BOTTOM);
	gtk_widget_set_size_request(view, 300, 200);
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_WORD_CHAR);
#ifdef GTKSOURCEVIEW
	gtk_source_buffer_set_highlight(buffer, TRUE);
	gtk_source_buffer_set_check_brackets(buffer, FALSE);
#endif

	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "New post");
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(
				GTK_TEXT_VIEW(view)),
			"em", "style", PANGO_STYLE_ITALIC, NULL);
	gtk_text_buffer_create_tag(gtk_text_view_get_buffer(
				GTK_TEXT_VIEW(view)),
			"strong", "weight", PANGO_WEIGHT_BOLD, NULL);

	gtk_text_view_set_editable(GTK_TEXT_VIEW(view), TRUE);
	gtk_container_add(GTK_CONTAINER(window), vbox);
	gtk_box_pack_start(GTK_BOX(vbox), entry, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			view, gtk_label_new("WYSIWYG"));

	source_view = 
#ifdef GTKSOURCEVIEW
			gtk_source_view_new_with_buffer(buffer),
#else
			gtk_text_view_new_with_buffer(buffer),
#endif
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(source_view), GTK_WRAP_WORD_CHAR);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			source_view,
		       	gtk_label_new("Source"));
#ifdef HILDON
	hildon_window_add_toolbar(HILDON_WINDOW(window), GTK_TOOLBAR(toolbar));
#else
	gtk_box_pack_start(GTK_BOX(vbox), toolbar, FALSE, FALSE, 0);
#endif

	g_signal_connect(gtk_text_view_get_buffer(GTK_TEXT_VIEW(view)),
		       	"changed", G_CALLBACK(_magic), gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))); 
	g_signal_connect(buffer,
		       	"changed", G_CALLBACK(_more_magic), gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))); 

	g_object_set_data(G_OBJECT(window), "combo", combo);
	button = gtk_tool_button_new(NULL, "Send");
	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(button));
	g_object_set_data(G_OBJECT(window), "buffer", buffer);
	g_object_set_data(G_OBJECT(window),
			"title-entry", entry);
	g_signal_connect(button, "clicked", G_CALLBACK(_send), window);
	button = gtk_tool_button_new(NULL, "Img");
	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(button));
	g_signal_connect(button, "clicked", G_CALLBACK(_img), gtk_text_view_get_buffer(GTK_TEXT_VIEW(view)));
#ifdef GEOCLUE
	button = gtk_tool_button_new(NULL, "Geo");
	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(button));
	g_signal_connect(button, "clicked", G_CALLBACK(_geo), window);
#endif
#ifdef HILDON
	button = gtk_toggle_tool_button_new();
	_tool_button_set_icon_name(GTK_TOOL_BUTTON(button),
		       	"qgn_list_gene_bold");
#else
	button = gtk_toggle_tool_button_new_from_stock(GTK_STOCK_BOLD);
#endif
	g_object_set_data_full(G_OBJECT(button), "tag",
			g_strdup("strong"), g_free);
	g_object_set(button, "can-focus", FALSE, NULL);
	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(button));
	g_signal_connect(button, "toggled", G_CALLBACK(_omg), view);
	g_object_set_data(G_OBJECT(
				gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))),
			"strong", button);

#ifdef HILDON
	button = gtk_toggle_tool_button_new();
	_tool_button_set_icon_name(GTK_TOOL_BUTTON(button),
		       	"qgn_list_gene_italic");
#else
	button = gtk_toggle_tool_button_new_from_stock(GTK_STOCK_ITALIC);
#endif
	g_object_set_data_full(G_OBJECT(button), "tag", g_strdup("em"), g_free);
	g_object_set(button, "can-focus", FALSE, NULL);
	g_object_set_data(G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))),
			"source-buffer", buffer);
	g_object_set_data(G_OBJECT(gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))),
			"title-entry", entry);
	g_signal_connect(button, "toggled", G_CALLBACK(_omg), view);
	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(button));
	g_object_set_data(G_OBJECT(
				gtk_text_view_get_buffer(GTK_TEXT_VIEW(view))),
			"em", button);

	gtk_container_add(GTK_CONTAINER(combo_item), combo);
	gtk_container_add(GTK_CONTAINER(toolbar), GTK_WIDGET(combo_item));

	g_signal_connect(combo, "changed", G_CALLBACK(_combo_changed), window);

	g_signal_connect_after(gtk_text_view_get_buffer(GTK_TEXT_VIEW(view)),
			"insert-text", G_CALLBACK(_zui), view);
	g_signal_connect(gtk_text_view_get_buffer(GTK_TEXT_VIEW(view)), "mark-set", G_CALLBACK(mark_set), NULL);
	g_signal_connect(window, "delete-event", G_CALLBACK(gtk_main_quit), NULL);

	return window;
}

static void _combo_insert(BlogPost *post, GtkComboBox *combo)
{
	gtk_combo_box_append_text(combo, post->title);
}

void update_posts(GtkWidget *window, guint active)
{
	GtkComboBox *combo = g_object_get_data(G_OBJECT(window), "combo");
	GtkTreeModel *model = gtk_combo_box_get_model(combo);
	Blog *blog = g_object_get_data(G_OBJECT(window), "blog");
	GtkWidget *cats = g_object_get_data(G_OBJECT(window), "categories");

	if (GTK_IS_LIST_STORE(model)) {
		gtk_list_store_clear(GTK_LIST_STORE(model));
	} else if (GTK_IS_TREE_STORE(model)) {
		gtk_tree_store_clear(GTK_TREE_STORE(model));
	} else {
		return;
	}

	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), "New post");

	if (!blog || !blog->posts) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
		return;
	}

	g_slist_foreach(blog->posts, (GFunc)_combo_insert, combo);

	if (cats) {
		if (blog->categories) {
			GtkWidget *menu = gtk_menu_new();
			GSList *iter = NULL;
			for (iter = blog->categories; iter; iter = iter->next) {
				GtkWidget *check_item =
					gtk_check_menu_item_new_with_label(
							((BlogCategory *)
							 iter->data)->name);
				g_object_set_data(G_OBJECT(check_item),
					       	"category",
						iter->data);

				gtk_menu_shell_append(GTK_MENU_SHELL(menu),
						check_item);
			}
			gtk_widget_show_all(menu);
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(cats), menu);
			gtk_widget_set_sensitive(cats, TRUE);
		} else {
			gtk_menu_item_set_submenu(GTK_MENU_ITEM(cats), NULL);
			gtk_widget_set_sensitive(cats, FALSE);
		}
	}
		
	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), active);
}
