#include "accounts.h"
#include "login.h"

struct LoginDialog {
	GtkWidget *combo;
	GtkWidget *entry;
};

enum Responses {
	RESPONSE_OK,
	RESPONSE_CANCEL,
	RESPONSE_ACCOUNTS,
	RESPONSE_OFFLINE
};

static void _cmb_changed(GtkComboBox *combo, struct LoginDialog *ddata)
{
	GtkTreeIter iter;
	gchar *pwd = NULL;

	if (gtk_combo_box_get_active_iter(combo, &iter)) {
		GtkTreeModel *model = gtk_combo_box_get_model(combo);
	
		gtk_tree_model_get(model, &iter, FIELD_PASSWORD, &pwd, -1);
	}
	gtk_entry_set_text(GTK_ENTRY(ddata->entry), pwd ? pwd : "");
	g_free(pwd);
}

GtkWindow *login_dialog(GtkWindow *parent, GtkTreeModel *accounts)
{
	GtkWidget *dialog;
	GtkCellRenderer *rend;
	struct LoginDialog *ddata = g_new(struct LoginDialog, 1);

	dialog = gtk_dialog_new_with_buttons("Login",
			parent,
			GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
			"Ok", RESPONSE_OK,
			"Accounts", RESPONSE_ACCOUNTS,
			"Offline", RESPONSE_OFFLINE,
			"Cancel", RESPONSE_CANCEL,
			NULL);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			gtk_label_new("Account:"));

	ddata->combo = gtk_combo_box_new_with_model(accounts);
	rend = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(ddata->combo),
			rend,
			TRUE);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(ddata->combo),
			rend,
			(GtkCellLayoutDataFunc)account_name,
			NULL,
			NULL);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			ddata->combo);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			gtk_label_new("Password:"));

	ddata->entry = gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(ddata->entry), FALSE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),
			ddata->entry);
	g_object_set_data_full(G_OBJECT(dialog), "private", ddata, g_free);
	if (model_has_data(accounts)) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(ddata->combo), 0);
	}
	g_signal_connect(ddata->combo, "changed",
			G_CALLBACK(_cmb_changed), ddata);
	_cmb_changed(GTK_COMBO_BOX(ddata->combo), ddata);


	return GTK_WINDOW(dialog);
}

void login_dialog_set_model(GtkWindow *login_dialog, GtkTreeModel *accounts)
{
	struct LoginDialog *ddata = g_object_get_data(G_OBJECT(login_dialog),
			"private");
	gtk_combo_box_set_model(GTK_COMBO_BOX(ddata->combo), accounts);
	if (model_has_data(accounts)) {
		gtk_combo_box_set_active(GTK_COMBO_BOX(ddata->combo), 0);
	}
	_cmb_changed(GTK_COMBO_BOX(ddata->combo), ddata);
}

dialogval login_dialog_run(GtkWindow *login_dialog, GtkTreeIter *iter)
{
	struct LoginDialog *ddata = g_object_get_data(G_OBJECT(login_dialog),
			"private");
	switch (gtk_dialog_run(GTK_DIALOG(login_dialog))) {
	case RESPONSE_OK:
		if (iter) {
			gtk_combo_box_get_active_iter(GTK_COMBO_BOX(ddata->combo), iter);
		}
		return gtk_combo_box_get_active(GTK_COMBO_BOX(ddata->combo));
	case RESPONSE_CANCEL:
		return CANCEL;
	case RESPONSE_ACCOUNTS:
		return ACCOUNTS;
	case RESPONSE_OFFLINE:
		return OFFLINE;
	}

	return CANCEL;
}
