#include <glib.h>
#include <gtk/gtk.h>
#include <string.h>
#include <gconf/gconf-client.h>
#include <stdlib.h>
#include "accounts.h"
#include "blog-plugin.h"
#include "plugin-loader.h"
#include "gui.h"
#include "login.h"

struct Foo {
	GtkWidget *dialog;
	GtkWidget *pbar;
	Blog ***blogs;
	guint blog;
	gboolean connected;
};

static gboolean _gtk_widget_destroy(gpointer data)
{
	gtk_widget_destroy(GTK_WIDGET(data));
	return FALSE;
}
/*
static void _g_source_remove(gpointer data,
		GObject *gone)
{
	(void)gone;
	g_source_remove(GPOINTER_TO_UINT(data));
}
*/
/*static gboolean _pulse(gpointer data)
{
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(data));
	return TRUE;
} */

static void gotrec(Blog *blog, const BlogPluginError *error, gpointer user_data)
{
	(void)blog;
	(void)error;
	struct Foo *foo = user_data;
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(foo->pbar), 1.00);
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(foo->pbar), "Done.");
	g_timeout_add(200, _gtk_widget_destroy, foo->dialog);
}

static void gotcat(Blog *blog, const BlogPluginError *error, gpointer user_data)
{
	struct Foo *foo = user_data;
	(void)error;

	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(foo->pbar), "Getting recent posts...");
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(foo->pbar), 0.66);
	blog->plugin->getRecent(blog, gotrec, foo);
}

static void connected(Blog **blogs, const BlogPluginError *error, gpointer user_data)
{
	struct Foo *foo = user_data;
	guint i;
	(void)error;

	*foo->blogs = blogs;
	foo->connected = TRUE;
	if (blogs[0] && blogs[1]) {
		GtkWidget *combo = gtk_combo_box_new_text();
		GtkWidget *dialog = gtk_dialog_new_with_buttons(
				"Insert details",
				GTK_WINDOW(foo->dialog),
				GTK_DIALOG_MODAL |
				GTK_DIALOG_NO_SEPARATOR,
				"OK", GTK_RESPONSE_OK,
				"Cancel", GTK_RESPONSE_CANCEL,
				NULL
				);

		for (i = 0; blogs[i]; i++) {
			gtk_combo_box_append_text(GTK_COMBO_BOX(combo),
					blogs[i]->name);
		}
		gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);

		gtk_container_add(GTK_CONTAINER(
					GTK_DIALOG(dialog)->vbox),
				combo);
		gtk_widget_show_all(dialog);

		if (gtk_dialog_run(GTK_DIALOG(dialog)) ==
				GTK_RESPONSE_OK) {
			foo->blog = gtk_combo_box_get_active(GTK_COMBO_BOX(
						combo));
		}
		gtk_widget_destroy(dialog);
	}
	gtk_progress_bar_set_text(GTK_PROGRESS_BAR(foo->pbar), "Getting categories...");
	gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(foo->pbar), 0.33);
	blogs[foo->blog]->plugin->getCategories(blogs[foo->blog], gotcat, foo);
}

int main(int argc, char **argv)
{
	GtkWidget *window;
	GtkWidget *dialog;
	/*
	GtkWidget *hbox;
	GtkWidget *entry1, *entry2, *entry3;
	GtkWidget *combo;
	GConfClient *gc;
	GtkCellRenderer *rend;
	gchar *temp; */
	BlogPlugin **plugins = NULL;
/*	guint blog = 0, i; */
	GtkListStore *store;
	GtkTreeIter dummy;

	gtk_init(&argc, &argv);

/*	gc = gconf_client_get_default(); */

	window = build_ui();
	gtk_widget_show_all(window);

	plugins = load_plugins();
	store = accounts_load(plugins);
	dialog = GTK_WIDGET(login_dialog(GTK_WINDOW(window), GTK_TREE_MODEL(store)));
	gtk_widget_show_all(dialog);
	if (!gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &dummy)) {
		if (accounts_dialog(GTK_WINDOW(window), store, plugins)) {
			accounts_save(store);
		} else {
			g_object_unref(store);
			store = accounts_load(plugins);
		}
		login_dialog_set_model(GTK_WINDOW(dialog), GTK_TREE_MODEL(store));
	}

	while (TRUE) {
		GtkTreeIter iter;
		dialogval val = login_dialog_run(GTK_WINDOW(dialog), &iter);
		if (val == CANCEL) {
			return 0;
		} else if (val == ACCOUNTS) {
			if (accounts_dialog(GTK_WINDOW(window), store, plugins)) {
				accounts_save(store);
			} else {
				g_object_unref(store);
				store = accounts_load(plugins);
			}
			login_dialog_set_model(GTK_WINDOW(dialog), GTK_TREE_MODEL(store));
		} else if (val == OFFLINE) {
			return 0;
		} else {
			GtkWidget *dlg = gtk_dialog_new_with_buttons(
					"Loading...",
					GTK_WINDOW(dialog),
					GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
					NULL);
			struct Foo bar;

			BlogPlugin *plugin;
			gchar *url;
			gchar *user;
			gchar *pass;
			Blog **blogs;

			bar.connected = 0;
			bar.blogs = &blogs;
			bar.blog = 0;
			bar.dialog = dlg;
			bar.pbar = gtk_progress_bar_new();

			g_signal_connect(dlg, "delete-event", G_CALLBACK(gtk_true), NULL);

			gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dlg)->vbox),
					bar.pbar);
			gtk_progress_bar_set_fraction(GTK_PROGRESS_BAR(bar.pbar), 0.0);
			gtk_progress_bar_set_text(GTK_PROGRESS_BAR(bar.pbar), "Connecting...");

			gtk_tree_model_get(GTK_TREE_MODEL(store),
					&iter,
					FIELD_URL, &url,
					FIELD_USER, &user,
					FIELD_PASSWORD, &pass,
					FIELD_PLUGIN, &plugin,
					-1);
			plugin->connect(plugin, url, user, pass, connected, &bar);
			if (!bar.connected) {
/*				guint tid = g_timeout_add(200, _pulse, bar.pbar);
				g_object_weak_ref(G_OBJECT(bar.pbar), _g_source_remove, GUINT_TO_POINTER(tid)); */
				gtk_widget_show_all(dlg);
				gtk_dialog_run(GTK_DIALOG(dlg));
			}
			g_object_set_data(G_OBJECT(window), "blog",
					blogs[bar.blog]);
			update_posts(window, 0);
			gtk_window_set_title(GTK_WINDOW(window), blogs[bar.blog]->name);
			gtk_widget_destroy(dialog);

			gtk_main();
			break;
		}
	}

	/*
	dialog = gtk_dialog_new_with_buttons("Insert details",
			GTK_WINDOW(window),
			GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
			"OK", GTK_RESPONSE_OK,
			"Cancel", GTK_RESPONSE_CANCEL,
			NULL
			);
	hbox = gtk_hbox_new(0, FALSE);
	gtk_container_add(GTK_CONTAINER(hbox), gtk_label_new("URI"));
	entry1 = gtk_entry_new();
	temp = gconf_client_get_string(gc, "/apps/maemo/blog/uri", NULL);
	if (temp) {
		gtk_entry_set_text(GTK_ENTRY(entry1), temp);
		g_free(temp);
	}
	gtk_container_add(GTK_CONTAINER(hbox), entry1);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbox);
	hbox = gtk_hbox_new(0, FALSE);
	gtk_container_add(GTK_CONTAINER(hbox), gtk_label_new("User"));
	entry2 = gtk_entry_new();
	temp = gconf_client_get_string(gc, "/apps/maemo/blog/user", NULL);
	if (temp) {
		gtk_entry_set_text(GTK_ENTRY(entry2), temp);
		g_free(temp);
	}
	gtk_container_add(GTK_CONTAINER(hbox), entry2);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbox);
	hbox = gtk_hbox_new(0, FALSE);
	gtk_container_add(GTK_CONTAINER(hbox), gtk_label_new("Password"));
	entry3 = gtk_entry_new();
	gtk_entry_set_visibility(GTK_ENTRY(entry3), FALSE);
	gtk_container_add(GTK_CONTAINER(hbox), entry3);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), hbox);

	combo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(store));
	rend = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo),
			rend,
			TRUE);
	gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(combo),
			rend,
			(GtkCellLayoutDataFunc)account_name,
			NULL,
			NULL);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), combo);

	gtk_widget_show_all(window);
	gtk_widget_show_all(dialog);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK) {
		guint plugin = gtk_combo_box_get_active(GTK_COMBO_BOX(combo));
		Blog **blogs = plugins[plugin]->connect(plugins[plugin],
				gtk_entry_get_text(GTK_ENTRY(entry1)),
				gtk_entry_get_text(GTK_ENTRY(entry2)),
				gtk_entry_get_text(GTK_ENTRY(entry3)));
		gconf_client_set_string(gc, "/apps/maemo/blog/uri",
				gtk_entry_get_text(GTK_ENTRY(entry1)),
				NULL);
		gconf_client_set_string(gc, "/apps/maemo/blog/user",
				gtk_entry_get_text(GTK_ENTRY(entry2)),
				NULL);
		gconf_client_set_string(gc, "/apps/maemo/blog/plugin",
				plugins[plugin]->name,
				NULL);
		gtk_widget_destroy(dialog);

		if (blogs[0] && blogs[1]) {
			combo = gtk_combo_box_new_text();
			dialog = gtk_dialog_new_with_buttons("Insert details",
					GTK_WINDOW(window),
					GTK_DIALOG_MODAL |
				       	GTK_DIALOG_NO_SEPARATOR,
					"OK", GTK_RESPONSE_OK,
					"Cancel", GTK_RESPONSE_CANCEL,
					NULL
					);

			for (i = 0; blogs[i]; i++) {
				gtk_combo_box_append_text(GTK_COMBO_BOX(combo),
						blogs[i]->name);
			}
			gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);

			gtk_container_add(GTK_CONTAINER(
						GTK_DIALOG(dialog)->vbox),
					combo);
			gtk_widget_show_all(dialog);

			if (gtk_dialog_run(GTK_DIALOG(dialog)) ==
					GTK_RESPONSE_OK) {
				blog = gtk_combo_box_get_active(GTK_COMBO_BOX(
							combo));
			} else {
				return 0;
			}
		}

		blogs[blog]->plugin->getCategories(blogs[blog]);
		blogs[blog]->plugin->getRecent(blogs[blog]);
		g_object_set_data(G_OBJECT(window), "blog",
				blogs[blog]);
		update_posts(window, 0);
		gtk_window_set_title(GTK_WINDOW(window), blogs[blog]->name);

		gtk_main();

	}
	*/

	return 0;
}
