#include <gmodule.h>
#include <string.h>
#include "blog-plugin.h"
#include "plugin-loader.h"

BlogPlugin **load_plugins(void)
{
	GDir *dir = g_dir_open(PLUGINDIR,
			0, NULL);
	GSList *list = NULL;
	const gchar *file;

	if (dir) {
		while ((file = g_dir_read_name(dir))) {
			gchar *full = g_build_filename(PLUGINDIR,
					file, NULL);
			GModule *mod = g_module_open(full, G_MODULE_BIND_LAZY); 
			if (mod) {
				BlogPlugin *(*init)(void);
				BlogPlugin *plugin;
				g_module_symbol(mod, 
						"plugin_init",
						(gpointer *)&init);
				plugin = init();
				if (plugin) {
					plugin->plugin_handle = mod;
					list = g_slist_append(list,
							plugin);
				}
			}
			g_free(full);
		}
		g_dir_close(dir);
	}

	if (list) {
		BlogPlugin **retval = g_new0(BlogPlugin *, g_slist_length(
					list) + 1);
		GSList *iter;
		guint i = 0;
		for (iter = list; iter; iter = iter->next) {
			retval[i] = (BlogPlugin *)iter->data;
			i++;
		}
		g_slist_free(list);
		
		return retval;
	}

	return NULL;
}

void unload_plugins(BlogPlugin **plugins)
{
	guint i;

	for (i = 0; plugins[i]; i++) {
		GModule *mod = (GModule *)plugins[i]->plugin_handle;
		plugins[i]->unLoad(plugins[i]);
		g_module_close(mod);
	}

	g_free(plugins);
}

gint plugin_id_by_name(BlogPlugin **plugins, const gchar *name)
{
	gint i;
	for (i = 0; plugins[i]; i++) {
		if (strcmp(plugins[i]->name, name) == 0) {
			return i;
		}
	}
	return -1;
}
